#include <argp.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "common.h"
#include "tipc.h"
#include "echo-srv-app.h"


static char doc[] = "Echo CA sample";

static char args_doc[] = "max-size";

static struct argp_option options[] = {
        {"max-size", 'm', "Number", 0, "Max size of transfered data."},
        { 0 }
};

struct arguments {
	int max_size;
	char *file;
};

static error_t parse_opt(int key, char *arg, struct argp_state *state) {
        struct arguments *argus = state->input;

        switch (key) {
        case 'm':
                argus->max_size = (int)strtol(arg,NULL,10);
                break;
        case ARGP_KEY_ARG:
                if (state->argc >= 1)
			argp_usage(state);
        case ARGP_KEY_END:
                if (state->argc < 1)
                        argp_usage(state);
        default:
                return ARGP_ERR_UNKNOWN;
        }

        return 0;
}

static struct argp argp = { options, parse_opt, args_doc, doc };

void echo_srv_app_handler(float* data,int size) {
        
	/* Send Phase */
	int send_srv_fd;
        data_collect_srv_msg_t *msg;
        uint32_t msg_size = sizeof(data_collect_srv_msg_t);
	printf("\nBuiling tipc protocol...\n");
        send_srv_fd = tipc_connect(TIPC_DEFAULT_NODE, SMALL_TA_COLLECT_SRV_CHAL);
        //send_srv_fd = tipc_connect(TIPC_DEFAULT_NODE, TA_COLLECT_SRV_CHAL);
	printf("Finished!\n");
        if (send_srv_fd < 0) {
                LOG("ECHO_SRV: tipc connect fail.\n");
                return;
        }

        msg = calloc(1, msg_size);
        if (msg == NULL) {
                LOG("ECHO_SRV: memory alloc fail.\n");
                return;
        }


	int block_nums = 0; // block_nums helps to find the start location
	msg->cmd = DATA_COLLECT_START;
	msg->data_size = size;
	printf("send data size is:%d; float size is %ld\n",msg->data_size,sizeof(float));
	
	/* sending size control */
	if (size > MAX_SIZE){
		printf("data size is too big, which is : %d\n",size);
		return;
	}

	printf("start to send data!\n");
	while(size>BLOCK_SIZE*block_nums){
		memcpy(msg->data, data+(BLOCK_SIZE*block_nums),BLOCK_SIZE);
        	/* Send msg to TA */
		printf("This is the %d round of sending data with size %d.\n",block_nums++,BLOCK_SIZE);
		write(send_srv_fd, msg, msg_size);
		read(send_srv_fd, msg, msg_size);
	}

	/* Send signal of end of data sending phase */
	printf("After %d round, sending phase is completed!\n",block_nums);
	msg->cmd = DATA_COLLECT_END;
	write(send_srv_fd,msg,msg_size);
	read(send_srv_fd, msg, msg_size);
	
	
	/* Receive Phase */
	block_nums = 0;
	while((msg->cmd == DATA_RETURN_START) && (BLOCK_SIZE*block_nums < size)){
		write(send_srv_fd,msg,msg_size);
		read(send_srv_fd, msg, msg_size);
		memcpy(data+(BLOCK_SIZE*block_nums),msg->data,BLOCK_SIZE);
		block_nums++;
	}

	printf("=========close echo communication=========\n");
	free(msg);
}


int main(int argc, char *argv[]) {

	struct arguments argus = {
		.max_size = 2048,
	};

        /* Handle the input parameters */
        argp_parse(&argp, argc, argv, 0, 0, &argus);

	/* Generate a simple float array */
	float data[argus.max_size];
	for(int i =0; i< argus.max_size; i++){
		data[i] = i*0.5;
	}
	printf("Input data size ; %d\n",argus.max_size);
        echo_srv_app_handler(data,argus.max_size);
	printf("END\n");
        return 0;
}
